UPDATE CMS_Query
SET QueryText = 
'SELECT ##TOPN##  IsSecuredNode, DocumentStylesheetID, DocumentPageTemplateID, NodeDocType, NodeHeadTags, NodeCacheMinutes, DocumentPageTitle, DocumentPageKeyWords, DocumentPageDescription, NodeBodyElementAttributes, RequiresSSL, NodeID, DocumentID, DocumentCulture, NodeACLID, NodeLinkedNodeID, ClassName, NodeAliasPath FROM View_CMS_Tree_Joined WHERE ##WHERE## ORDER BY ##ORDERBY##' WHERE QueryName = 'selectuppertree' AND ClassID = (SELECT ClassID FROM CMS_Class WHERE ClassName=N'CMS.Document')
GO

-- Inserting Resource strings
DECLARE @stringId AS int;
DECLARE @cultureId AS int;

-- Get culture ID
SET @cultureID = (SELECT UICultureID FROM [CMS_UICulture] WHERE UICultureCode='en-US');

-- Check if resource string exists
SET @stringId = (SELECT StringID FROM [CMS_ResourceString] LEFT JOIN [CMS_ResourceTranslation] ON StringID=TranslationStringID WHERE StringKey = N'UserControlSelector.RequireFileName' AND TranslationUICultureID = @cultureID);
IF  @stringId IS NULL
BEGIN
-- Insert resource string
INSERT INTO [CMS_ResourceString]
           ([StringKey]
           ,[StringIsCustom]
           ,[StringLoadGeneration])
     VALUES
           ('UserControlSelector.RequireFileName'
           ,0
           ,0)

SET @stringId = scope_identity();

-- Insert translation
INSERT INTO [CMS_ResourceTranslation]
           ([TranslationStringID]
           ,[TranslationUICultureID]
           ,[TranslationText])
     VALUES
           (@stringId
           ,@cultureId
           ,'Please enter some file name.')
END

-- Check if resource string exists
SET @stringId = (SELECT StringID FROM [CMS_ResourceString] LEFT JOIN [CMS_ResourceTranslation] ON StringID=TranslationStringID WHERE StringKey = N'UserControlSelector.RequireFolderName' AND TranslationUICultureID = @cultureID);
IF  @stringId IS NULL
BEGIN
-- Insert resource string
INSERT INTO [CMS_ResourceString]
           ([StringKey]
           ,[StringIsCustom]
           ,[StringLoadGeneration])
     VALUES
           ('UserControlSelector.RequireFolderName'
           ,0
           ,0)

SET @stringId = scope_identity();

-- Insert translation
INSERT INTO [CMS_ResourceTranslation]
           ([TranslationStringID]
           ,[TranslationUICultureID]
           ,[TranslationText])
     VALUES
           (@stringId
           ,@cultureId
           ,'Please enter some folder name.')
END

-- Check if resource string exists
SET @stringId = (SELECT StringID FROM [CMS_ResourceString] LEFT JOIN [CMS_ResourceTranslation] ON StringID=TranslationStringID WHERE StringKey = N'dialogs.filesystem.ExcludedExtension' AND TranslationUICultureID = @cultureID);
IF  @stringId IS NULL
BEGIN
-- Insert resource string
INSERT INTO [CMS_ResourceString]
           ([StringKey]
           ,[StringIsCustom]
           ,[StringLoadGeneration])
     VALUES
           ('dialogs.filesystem.ExcludedExtension'
           ,0
           ,0)

SET @stringId = scope_identity();

-- Insert translation
INSERT INTO [CMS_ResourceTranslation]
           ([TranslationStringID]
           ,[TranslationUICultureID]
           ,[TranslationText])
     VALUES
           (@stringId
           ,@cultureId
           ,'Entered file name extension is in list of excluded extensions: %%extensions%%, please enter filename with other extension.')
END

-- Check if resource string exists
SET @stringId = (SELECT StringID FROM [CMS_ResourceString] LEFT JOIN [CMS_ResourceTranslation] ON StringID=TranslationStringID WHERE StringKey = N'dialogs.filesystem.NotAllowedExtension' AND TranslationUICultureID = @cultureID);
IF  @stringId IS NULL
BEGIN
-- Insert resource string
INSERT INTO [CMS_ResourceString]
           ([StringKey]
           ,[StringIsCustom]
           ,[StringLoadGeneration])
     VALUES
           ('dialogs.filesystem.NotAllowedExtension'
           ,0
           ,0)

SET @stringId = scope_identity();

-- Insert translation
INSERT INTO [CMS_ResourceTranslation]
           ([TranslationStringID]
           ,[TranslationUICultureID]
           ,[TranslationText])
     VALUES
           (@stringId
           ,@cultureId
           ,'Please enter a file name with an allowed extension. Allowed extensions: %%extensions%%.')
END

-- Check if resource string exists
SET @stringId = (SELECT StringID FROM [CMS_ResourceString] LEFT JOIN [CMS_ResourceTranslation] ON StringID=TranslationStringID WHERE StringKey = N'dialogs.filesystem.NotAllowedFolder' AND TranslationUICultureID = @cultureID);
IF  @stringId IS NULL
BEGIN
-- Insert resource string
INSERT INTO [CMS_ResourceString]
           ([StringKey]
           ,[StringIsCustom]
           ,[StringLoadGeneration])
     VALUES
           ('dialogs.filesystem.NotAllowedFolder'
           ,0
           ,0)

SET @stringId = scope_identity();

-- Insert translation
INSERT INTO [CMS_ResourceTranslation]
           ([TranslationStringID]
           ,[TranslationUICultureID]
           ,[TranslationText])
     VALUES
           (@stringId
           ,@cultureId
           ,'Entered folder name path or its part is not allowed.')
END

GO

-- Insert new DeleteArchived query 
DECLARE @classID int
SET @classID = (SELECT ClassID FROM CMS_Class WHERE ClassName LIKE 'cms.email')

IF  (SELECT TOP 1 QueryName FROM [CMS_Query] WHERE ClassID = @classID AND QueryName LIKE 'deletearchived') IS NULL
BEGIN

	INSERT INTO [CMS_Query]
				(
				[QueryName],
				[QueryTypeID],
				[QueryText],
				[QueryRequiresTransaction],
				[ClassID],
				[QueryIsLocked],
				[QueryLastModified],
				[QueryGUID],
				[QueryLoadGeneration]
				)
		  VALUES
				(
				 'deletearchived',
				 1,
				 'Proc_CMS_Email_DeleteArchived',
				 0,
				 @classID,
				 1,
				 getDate(),
				 newId(),
				 2
				)
END				
GO

-- Check if Stored Procedure already exists
IF (SELECT TOP 1 Name FROM SysObjects WHERE Type = 'P' and Name='Proc_CMS_Email_DeleteArchived') IS NOT NULL
BEGIN
DROP PROCEDURE [Proc_CMS_Email_DeleteArchived]
END

SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
CREATE PROCEDURE [Proc_CMS_Email_DeleteArchived]
	-- Procedure parameters
	@SiteID int	= 0,
	@ExpiryDate datetime,
	@EmailStatus int = 3
AS
BEGIN
SET NOCOUNT ON;
	-- Declare table for emails
	DECLARE @emails TABLE ( EmailID int not null ) 
	BEGIN
		-- Get global e-mails
		IF ((@SiteID = 0) OR (@SiteID IS NULL))
			INSERT INTO @emails 
				SELECT EmailID FROM CMS_Email 
				WHERE EmailLastSendAttempt <= @ExpiryDate
					AND (EmailSiteID IS NULL)
					AND EmailStatus = @EmailStatus
		-- Get all e-mails attached to the site			
		ELSE                   
			INSERT INTO @emails 
				SELECT EmailID FROM CMS_Email 
				WHERE EmailLastSendAttempt <= @ExpiryDate
					AND EmailSiteID = @SiteID
					AND EmailStatus = @EmailStatus
	END
      
	BEGIN TRANSACTION
          
	-- Delete attachment binding 
	DELETE FROM [CMS_AttachmentForEmail] WHERE EmailID IN (SELECT EmailID FROM @emails)
	-- Delete user binding
	DELETE FROM [CMS_EmailUser] WHERE EmailID IN (SELECT EmailID FROM @emails)
	-- Delete all attachments that have no bindings
	DELETE FROM CMS_EmailAttachment WHERE AttachmentID NOT IN (SELECT DISTINCT AttachmentID FROM CMS_AttachmentForEmail)
	-- Delete e-mails
	DELETE FROM [CMS_Email] WHERE EmailID IN (SELECT EmailID FROM @emails)
      
	COMMIT TRANSACTION;
END
GO

